package fr.username404.snowygui.gui

import com.mojang.blaze3d.vertex.BufferBuilder
import com.mojang.blaze3d.vertex.PoseStack
import com.mojang.blaze3d.vertex.Tesselator
import com.mojang.blaze3d.vertex.VertexConsumer
import fr.username404.snowygui.Snowy

fun interface Renderable { fun render(poseStack: PoseStack?) }
abstract class Element(
    @JvmField val xOrigin: Double, @JvmField val yOrigin: Double,
    val width: Int, val height: Int
): Renderable {
    var x = xOrigin; var y = yOrigin
    internal fun withinBounds(coordinateX: Double, coordinateY: Double): Boolean =
        ((coordinateX >= this.x) && (coordinateX <= (this.x + this.width))) and ((coordinateY >= this.y) && (coordinateY <= (this.y + this.height)))
    companion object Rendering {
        private var caughtError: Boolean = false
        @JvmStatic protected val tessellator: Tesselator = Tesselator.getInstance()
        @JvmStatic protected val buffer: BufferBuilder = tessellator.builder
    }
    fun display(stack: PoseStack? = null) {
        if (!hidden && !caughtError) try {
            render(stack)
        } catch (t: Throwable) {
            with(Snowy.logs) {
                error("An element from snowy threw an error: \n\t$t")
                debug("\t${t.stackTraceToString()}")
                warn("Rendering of snowy UI elements will now be disabled to avoid further errors.")
                caughtError = true
            }
        }
    }
    var hidden: Boolean = false; protected set
}

abstract class ColoredElement(x: Double, y: Double, width: Int, height: Int, private val color: Int, private val opacity: Float) : Element(x, y, width, height) {
    companion object {
        const val TransparentColor: Int = -0x1
        @JvmStatic protected fun VertexConsumer.colorIt(color: Int, opacity: Float = 1F): VertexConsumer {
            with(hextoRGB(color)) {
                return this@colorIt.color(get(0), get(1), get(2), opacity)
            }
        }
    }
    protected fun VertexConsumer.colorEnd(color: Int = this@ColoredElement.color) = colorIt(color, opacity).endVertex()
}

fun hextoRGB(hex: Int): MutableList<Float> {
    val returnedColorList = mutableListOf<Float>()
    for (i in 2 downTo 0) {
        returnedColorList.add(
            (hex shr (i * 8) and 255) / 255.0F
        )
    }
    return returnedColorList
}